@@ -22,7 +22,14 @@ test_that("string vectors with only empty strings and nulls don't allocate a dat
   expect_equal(a$length(), 1L)
 
   buffers <- a$data()$buffers
-  expect_null(buffers[[1]])
-  expect_null(buffers[[3]])
+
+  # No nulls
+  expect_equal(buffers[[1]]$size, 1)
+
+  # Offsets has 2 elements
   expect_equal(buffers[[2]]$size, 8L)
+
+  # As per ARROW-2744, values buffer should preferably be non-null.
+  expect_equal(buffers[[3]]$size, 0L)
+  expect_equal(buffers[[3]]$capacity, 0L)
 })
