@@ -21,7 +21,7 @@ dst_dir <- paste0("libarrow/arrow-", VERSION)
 
 arrow_repo <- "https://dl.bintray.com/ursalabs/arrow-r/libarrow/"
 apache_src_url <- paste0(
-  "https://www.apache.org/dyn/closer.cgi/arrow/arrow-", VERSION,
+  "https://archive.apache.org/dist/arrow/arrow-", VERSION,
   "/apache-arrow-", VERSION, ".tar.gz"
 )
 
@@ -29,20 +29,11 @@ options(.arrow.cleanup = character()) # To collect dirs to rm on exit
 on.exit(unlink(getOption(".arrow.cleanup")))
 
 env_is <- function(var, value) identical(tolower(Sys.getenv(var)), value)
-
-# This condition identifies when you're installing locally, either by presence
-# of NOT_CRAN or by absence of a `check` env var. So even if you don't have
-# NOT_CRAN set in your dev environment, this will still pass outside R CMD check
-locally_installing <- env_is("NOT_CRAN", "true") || env_is("_R_CHECK_SIZE_OF_TARBALL_", "")
-# Combine with explicit vars to turn off downloading and building. I.e. only
-# downloads/builds when local, but can turn off either with these env vars.
 # * no download, build_ok: Only build with local git checkout
 # * download_ok, no build: Only use prebuilt binary, if found
 # * neither: Get the arrow-without-arrow package
-download_ok <- locally_installing && !env_is("LIBARROW_DOWNLOAD", "false")
-build_ok <- locally_installing && !env_is("LIBARROW_BUILD", "false")
-# TODO: allow LIBARROW_DOWNLOAD=true to override locally_installing? or just set NOT_CRAN?
-
+download_ok <- env_is("LIBARROW_DOWNLOAD", "true") || !(tolower(Sys.getenv("LIBARROW_BINARY")) %in% c("", "false"))
+build_ok <- !env_is("LIBARROW_BUILD", "false")
 # For local debugging, set ARROW_R_DEV=TRUE to make this script print more
 quietly <- !env_is("ARROW_R_DEV", "true")
 
@@ -67,20 +58,20 @@ download_binary <- function(os = identify_os()) {
 }
 
 # Function to figure out which flavor of binary we should download, if at all.
-# By default ("FALSE"), it will not download a precompiled library,
-# but you can override this by setting the env var LIBARROW_BINARY_DISTRO to:
+# By default (unset or "FALSE"), it will not download a precompiled library,
+# but you can override this by setting the env var LIBARROW_BINARY to:
 # * `TRUE` (not case-sensitive), to try to discover your current OS, or
 # * some other string, presumably a related "distro-version" that has binaries
 #   built that work for your OS
-identify_os <- function(os = Sys.getenv("LIBARROW_BINARY_DISTRO", "false")) {
-    if (identical(tolower(os), "false")) {
-      # Env var says not to download a binary
-      return(NULL)
-    } else if (!identical(tolower(os), "true")) {
-      # Env var provided an os-version to use--maybe you're on Ubuntu 18.10 but
-      # we only build for 18.04 and that's fine--so use what the user set
-      return(os)
-    }
+identify_os <- function(os = Sys.getenv("LIBARROW_BINARY", Sys.getenv("LIBARROW_DOWNLOAD"))) {
+  if (tolower(os) %in% c("", "false")) {
+    # Env var says not to download a binary
+    return(NULL)
+  } else if (!identical(tolower(os), "true")) {
+    # Env var provided an os-version to use--maybe you're on Ubuntu 18.10 but
+    # we only build for 18.04 and that's fine--so use what the user set
+    return(os)
+  }
 
   if (nzchar(Sys.which("lsb_release"))) {
     distro <- tolower(system("lsb_release -is", intern = TRUE))
